import { visit } from 'unist-util-visit';
import { toString } from 'hast-util-to-string';
import { fromString } from 'hast-util-from-string';

function addClass(node, className) {
    if (!node.properties)
        return;
    if (!node.properties.className) {
        if (Array.isArray(className))
            node.properties.className = className;
        else
            node.properties.className = [className];
    }
    if (Array.isArray(className))
        node.properties.className.push(...className);
    else
        node.properties.className.push(className);
}
const rehypePrismDiff = (option = {
    remove: false
}) => {
    const classMapping = Object.assign({
        diff: 'code-diff',
        deleted: 'diff-deleted',
        inserted: 'diff-inserted',
        warn: 'diff-warn',
        comment: 'diff-comment'
    }, option.classMapping || {});
    return (tree) => {
        visit(tree, 'element', (node, _index, parent) => {
            if (!parent || parent.tagName !== 'pre' || !('tagName' in node) || node.tagName !== 'code') {
                return;
            }
            const meta = node.data && node.data.meta ? node.data.meta : '';
            if (!meta.toLowerCase().includes('diff'.toLowerCase()))
                return;
            addClass(node, classMapping.diff);
            node.children.forEach((line) => {
                if (line.type !== 'element')
                    return;
                const lineString = toString(line);
                if (lineString.substring(0, 1) === '-') {
                    addClass(line, classMapping.deleted);
                }
                else if (lineString.substring(0, 1) === '+') {
                    addClass(line, classMapping.inserted);
                }
                else if (lineString.substring(0, 1) === '!') {
                    addClass(line, classMapping.warn);
                }
                else if (lineString.substring(0, 1) === '#') {
                    addClass(line, classMapping.comment);
                }
                if (option?.remove) {
                    removeFirstChar(line);
                }
            });
        });
    };
    function removeFirstChar(line) {
        line.children.forEach((item) => {
            if (item.type !== 'text')
                return;
            const itemString = toString(item);
            if (itemString.match(/\+|-|!|#/))
                fromString(line, itemString.substring(1));
        });
    }
};

export { rehypePrismDiff as default };
//# sourceMappingURL=index.js.map
